@@ -261,25 +261,6 @@ static int ends_in(const char *name, const char *ext)
 	return 0;
 }
 
-/* "\177ELF" <byte> where byte = 001 for 32-bit, 002 for 64 */
-int needconv(const char *elfhdr)
-{
-	union { short s; char c[2]; } endian_test;
-
-	endian_test.s = 1;
-	if (endian_test.c[1] == 1) return elfhdr[EI_DATA] != ELFDATA2MSB;
-	if (endian_test.c[0] == 1) return elfhdr[EI_DATA] != ELFDATA2LSB;
-	else
-		abort();
-}
-
-static char *my_basename(const char *name)
-{
-	const char *base = strrchr(name, '/');
-	if (base) return (char *)base + 1;
-	return (char *)name;
-}
-
 static struct module *grab_module(const char *dirname, const char *filename)
 {
 	struct module *new;
@@ -302,25 +283,23 @@ static struct module *grab_module(const char *dirname, const char *filename)
 		goto fail_data;
 	}
 
-	/* "\177ELF" <byte> where byte = 001 for 32-bit, 002 for 64 */
-	if (memcmp(new->data, ELFMAG, SELFMAG) != 0) {
-		warn("Module %s is not an elf object\n", new->pathname);
-		goto fail;
-	}
-
-	switch (((char *)new->data)[EI_CLASS]) {
+	switch (elf_ident(new->data, new->len, &new->conv)) {
 	case ELFCLASS32:
 		new->ops = &mod_ops32;
 		break;
 	case ELFCLASS64:
 		new->ops = &mod_ops64;
 		break;
-	default:
-		warn("Module %s has elf unknown identifier %i\n",
-		     new->pathname, ((char *)new->data)[EI_CLASS]);
+	case -ENOEXEC:
+		warn("Module %s is not an elf object\n", new->pathname);
 		goto fail;
-	}
-	new->conv = needconv(new->data);
+	case -EINVAL:
+		warn("Module %s has unknown endianness\n", new->pathname);
+		goto fail;
+ 	default:
+		warn("Module %s has unknown word size\n", new->pathname);
+ 		goto fail;
+ 	}
 	return new;
 
 fail:
@@ -1173,6 +1152,9 @@ int main(int argc, char *argv[])
 	int i;
 	const char *config = NULL;
 
+	if (native_endianness() == 0)
+		abort();
+
 	/* Don't print out any errors just yet, we might want to exec
            backwards compat version. */
 	opterr = 0;
