@@ -9,9 +9,6 @@ void add_symbol(const char *name, struct module *owner);
 struct module *find_symbol(const char *name, const char *modname, int weak);
 void add_dep(struct module *mod, struct module *depends_on);
 
-/* I hate strcmp. */
-#define streq(a,b) (strcmp((a),(b)) == 0)
-
 struct module
 {
 	/* Next module in list of all modules */
@@ -30,6 +27,9 @@ struct module
 	/* Set while we are traversing dependencies */
 	struct list_head dep_list;
 
+	/* Line number in modules.order (or INDEX_PRIORITY_MIN) */
+	unsigned int order;
+
 	/* Tables extracted from module by ops->fetch_tables(). */
 	unsigned int pci_size;
 	void *pci_table;
@@ -60,19 +60,4 @@ struct module
 	char pathname[0];
 };
 
-#define END(x, conv)							  \
-({									  \
-	typeof(x) __x;							  \
-	if (conv) __convert_endian(&(x), &(__x), sizeof(__x));		  \
-	else __x = (x);							  \
-	__x;								  \
-})
-
-static inline void __convert_endian(const void *src, void *dest,
-				    unsigned int size)
-{
-	unsigned int i;
-	for (i = 0; i < size; i++)
-		((unsigned char*)dest)[i] = ((unsigned char*)src)[size - i-1];
-}
 #endif /* MODINITTOOLS_DEPMOD_H */
