@@ -12,110 +12,15 @@
 #include <string.h>
 #include <sys/utsname.h>
 #include <sys/mman.h>
-#include "zlibsupport.h"
-#include "backwards_compat.c"
 
-#define streq(a,b) (strcmp((a),(b)) == 0)
-#define strstarts(a,start) (strncmp((a),(start), strlen(start)) == 0)
+#include "util.h"
+#include "zlibsupport.h"
+#include "testing.h"
 
 #ifndef MODULE_DIR
 #define MODULE_DIR "/lib/modules"
 #endif
 
-static int elf_endian;
-static int my_endian;
-
-static inline void __endian(const void *src, void *dest, unsigned int size)
-{
-	unsigned int i;
-	for (i = 0; i < size; i++)
-		((unsigned char*)dest)[i] = ((unsigned char*)src)[size - i-1];
-}
-
-#define TO_NATIVE(x)							  \
-({									  \
-	typeof(x) __x;							  \
-	if (elf_endian != my_endian) __endian(&(x), &(__x), sizeof(__x)); \
-	else __x = x;							  \
-	__x;								  \
-})
-
-static void *get_section32(void *file, unsigned long *size, const char *name)
-{
-	Elf32_Ehdr *hdr = file;
-	Elf32_Shdr *sechdrs = file + TO_NATIVE(hdr->e_shoff);
-	const char *secnames;
-	unsigned int i;
-
-	secnames = file
-		+ TO_NATIVE(sechdrs[TO_NATIVE(hdr->e_shstrndx)].sh_offset);
-	for (i = 1; i < TO_NATIVE(hdr->e_shnum); i++)
-		if (streq(secnames + TO_NATIVE(sechdrs[i].sh_name), name)) {
-			*size = TO_NATIVE(sechdrs[i].sh_size);
-			return file + TO_NATIVE(sechdrs[i].sh_offset);
-		}
-	return NULL;
-}
-
-static void *get_section64(void *file, unsigned long *size, const char *name)
-{
-	Elf64_Ehdr *hdr = file;
-	Elf64_Shdr *sechdrs = file + TO_NATIVE(hdr->e_shoff);
-	const char *secnames;
-	unsigned int i;
-
-	secnames = file
-		+ TO_NATIVE(sechdrs[TO_NATIVE(hdr->e_shstrndx)].sh_offset);
-	for (i = 1; i < TO_NATIVE(hdr->e_shnum); i++)
-		if (streq(secnames + TO_NATIVE(sechdrs[i].sh_name), name)) {
-			*size = TO_NATIVE(sechdrs[i].sh_size);
-			return file + TO_NATIVE(sechdrs[i].sh_offset);
-		}
-	return NULL;
-}
-
-static int elf_ident(void *mod, unsigned long size)
-{
-	/* "\177ELF" <byte> where byte = 001 for 32-bit, 002 for 64 */
-	char *ident = mod;
-
-	if (size < EI_CLASS || memcmp(mod, ELFMAG, SELFMAG) != 0)
-		return ELFCLASSNONE;
-	elf_endian = ident[EI_DATA];
-	return ident[EI_CLASS];
-}
-
-static void *get_section(void *file, unsigned long filesize,
-			 unsigned long *size, const char *name)
-{
-	switch (elf_ident(file, filesize)) {
-	case ELFCLASS32:
-		return get_section32(file, size, name);
-	case ELFCLASS64:
-		return get_section64(file, size, name);
-	default:
-		return NULL;
-	}
-}
-
-static const char *next_string(const char *string, unsigned long *secsize)
-{
-	/* Skip non-zero chars */
-	while (string[0]) {
-		string++;
-		if ((*secsize)-- <= 1)
-			return NULL;
-	}
-
-	/* Skip any zero padding. */
-	while (!string[0]) {
-		string++;
-		if ((*secsize)-- <= 1)
-			return NULL;
-	}
-	return string;
-}
-
 struct param
 {
 	struct param *next;
@@ -220,7 +125,7 @@ static struct option options[] =
 	{"version", 0, 0, 'V'},
 	{"help", 0, 0, 'h'},
 	{"null", 0, 0, '0'},
-	{"field", 0, 0, 'F'},
+	{"field", 1, 0, 'F'},
 	{0, 0, 0, 0}
 };
 
@@ -279,15 +184,16 @@ static void *grab_module(const char *name, unsigned long *size, char**filename,
 	struct utsname buf;
 	char *depname, *p, *moddir;
 
-	data = grab_file(name, size);
-	if (data) {
-		*filename = strdup(name);
-		return data;
-	}
-	if (errno != ENOENT) {
-		fprintf(stderr, "modinfo: could not open %s: %s\n",
-			name, strerror(errno));
-		return NULL;
+	if (strchr(name, '.') || strchr(name, '/')) {
+		data = grab_file(name, size);
+		if (data) {
+			*filename = strdup(name);
+			return data;
+		} else {
+			fprintf(stderr, "modinfo: could not open %s: %s\n",
+				name, strerror(errno));
+			return NULL;
+		}
 	}
 
 	if (kernel) {
@@ -366,7 +272,6 @@ static void usage(const char *name)
 
 int main(int argc, char *argv[])
 {
-	union { short s; char c[2]; } endian_test;
 	const char *field = NULL;
 	const char *kernel = NULL;
 	char sep = '\n';
@@ -374,13 +279,7 @@ int main(int argc, char *argv[])
 	int opt, ret = 0;
 	char *basedir = "";
 
-	if (!getenv("NEW_MODINFO"))
-		try_old_version("modinfo", argv);
-
-	endian_test.s = 1;
-	if (endian_test.c[1] == 1) my_endian = ELFDATA2MSB;
-	else if (endian_test.c[0] == 1) my_endian = ELFDATA2LSB;
-	else
+	if (native_endianness() == 0)
 		abort();
 
 	while ((opt = getopt_long(argc,argv,"adlpVhn0F:k:b:",options,NULL)) >= 0){
@@ -395,12 +294,15 @@ int main(int argc, char *argv[])
 		case '0': sep = '\0'; break;
 		case 'k': kernel = optarg; break;
 		case 'b': basedir = optarg; break;
+		case 'h': usage(argv[0]); exit(0); break;
 		default:
-			usage(argv[0]); exit(0);
+			usage(argv[0]); exit(1);
 		}
 	}
-	if (argc < optind + 1)
+	if (argc < optind + 1) {
 		usage(argv[0]);
+		exit(1);
+	}
 
 	for (opt = optind; opt < argc; opt++) {
 		void *info, *mod;
@@ -414,7 +316,7 @@ int main(int argc, char *argv[])
 			continue;
 		}
 
-		info = get_section(mod, modulesize, &infosize, ".modinfo");
+		info = get_section(mod, modulesize, ".modinfo", &infosize);
 		if (!info) {
 			release_file(mod, modulesize);
 			free(filename);
